function mm2in(n) {
    return n / 25.4
}

function in2mm(n) {
    return n * 25.4
}

function cm2in(n) {
    return n / 2.54
}

function in2cm(n) {
    return n * 2.54
}

function g2oz(n) {
    return n / 28.3495231
}

function oz2g(n) {
    return n * 28.3495231
}

function g2gr(n) {
    return n / 0.06479891
}

function gr2g(n) {
    return n * 0.06479891
}
var in2 = 25.4 * 25.4;
var in3 = 25.4 * in2;
var materials;

function awg2in(a) {
    return 0.005 * Math.pow(92, (36 - a) / 39)
}

function awg2mm(a) {
    return in2mm(awg2in(a))
}

function defaultCoil() {
    return {
    showSi: true,
    advanced: false,
    mat: 'ka1',
    profile: 'round',
    strands: 1,
    twistpitch: 0,
    awg: 30,
    setup: 's',
    targetr: 1.8,
    rstv: 0.028472674975859414,
    wirelength: 63.21850692027117,
    dia: 0.2546390029766585,
    width: 0.5,
    thickness: 0.1,
    innerdia: 1.3,
    leglength: 5,
    wrapspace: 0.01,
    useNaDia: false,
    hfnw: 10
    }
}
var coil = defaultCoil();
coil.converter = {};
coil.defaultCoil = defaultCoil();
var aux = {};

function findXArea(a) {
    if (a.profile === 'ribbon') {
        return a.width * a.thickness
    }
    return Math.PI * (a.dia / 2) * (a.dia / 2) * a.strands
}

function ds(c, a, b) {
    if (coil.showSi) {
        if (a === undefined) {
            a = 3
        }
        return round(c, a)
    }
    if (b === undefined) {
        b = 5
    }
    return round(mm2in(c), b)
}

function writeRstv(a) {
    if (a.showSi) {
        $('#cd-rstv').val(round(a.rstv, 5)).parents('.form-group')
    } else {
        $('#cd-rstv').val(round(in2mm(a.rstv), 4)).parents('.form-group')
    }
    return
}

function findMat(a) {
    if (a === 'c') {
        return {
        hc: 0.5,
        dens: 8.31
        }
    }
    var b;
    for (b = 0; b < materials.length; b++) {
        if (materials[b].id === a) {
            return materials[b]
        }
    }
    return
}

function findResistivity(a) {
    if (a.mat === 'c') {
        return
    }
    var b = findMat(a.mat).rstv / 1000;
    var c = b / findXArea(a);
    coil.rstv = c;
    if (coil === a) {
        writeRstv(a)
    }
    return
}

function getRfactor(a) {
    if (a.setup === 's' || a.setup === 'qsp') {
        return 1
    }
    if (a.setup === 'dp') {
        return 0.5
    }
    if (a.setup === 'tp') {
        return 1 / 3
    }
    if (a.setup === 'qp') {
        return 0.25
    }
    if (a.setup === '5p') {
        return 0.2
    }
    if (a.setup === '6p') {
        return 1 / 6
    }
    if (a.setup === '7p') {
        return 1 / 7
    }
    if (a.setup === '8p') {
        return 1 / 8
    }
    if (a.setup === 'ds') {
        return 2
    }
    if (a.setup === 'ts') {
        return 3
    }
    if (a.setup === 'qs') {
        return 4
    }
    console.log('警告: getRfactor(spole)运行失敗, 未知設置 [' + a.setup + '].');
    return
}

function findTwist(a) {
    if (a.twistpitch === 0) {
        a.twistfactor = 1
    } else if (a.strands > 1) {
        a.twistlead = a.twistpitch * a.strands;
        var b = a.dia;
        var c = b * Math.PI;
        a.twisthypotenus = Math.sqrt((c * c) + (a.twistlead * a.twistlead));
        a.twistfactor = (a.twisthypotenus / a.twistlead);
        a.rstv *= a.twistfactor;
        if (a === coil) {
            writeRstv(a)
        }
    } else {
        a.twistfactor = 1
    }
    return
}

function klon(o) {
    var k = {};
    var x;
    for (x in o) {
        if (o.hasOwnProperty(x)) {
            k[x] = o[x]
        }
    }
    return k
}

function wireLength(a) {
    a.wirelength = (a.targetr / a.rstv);
    a.wirelength /= getRfactor(a);
    if (coil === a) {
        $('#cd-len').val(ds(a.wirelength, 2, 3))
    }
    return
}

function exportSettings() {
    var b = defaultCoil();
    var c = {};
    
    function expIfNotDef(a) {
        if (coil[a] !== b[a]) {
            if (a === 'leglength') {
                c.ll = coil.leglength
            } else if (a === 'innerdia') {
                c.id = coil.innerdia
            } else if (a === 'targetr') {
                c.r = coil.targetr
            } else if (a === 'setup') {
                c.s = coil.setup
            } else if (a === 'profile') {
                c.p = coil.profile
            } else if (a === 'twistpitch') {
                c.tp = coil.twistpitch
            } else if (a === 'strands') {
                c.str = coil.strands
            } else if (a === 'wirelength') {
                c.wl = coil.wirelength
            } else if (a === 'wrapspace') {
                c.ws = coil.wrapspace
            } else if (a === 'hfnw') {
                c.hfnw = coil.hfnw
            } else {
                c[a] = coil[a]
            }
        }
        return
    }
    if (!coil.showSi) {
        c.si = false
    }
    if (coil.advanced) {
        c.a = true
    }
    if (coil.useNaDia) {
        c.useNaDia = true
    }
    if (coil.mat === 'c') {
        c.rstv = coil.rstv
    } else {
        expIfNotDef('mat')
    }
    expIfNotDef('profile');
    expIfNotDef('twistpitch');
    expIfNotDef('setup');
    expIfNotDef('targetr');
    expIfNotDef('hfnw');
    if (coil.profile === 'ribbon') {
        expIfNotDef('width');
        expIfNotDef('thickness')
    } else {
        expIfNotDef('strands');
        if (coil.awg === '') {
            expIfNotDef('dia')
        } else {
            expIfNotDef('awg')
        }
    }
    var d = klon(coil);
    findResistivity(d);
    wireLength(d);
    if (coil.wirelength !== d.wirelength) {
        c.wl = coil.wirelength
    }
    expIfNotDef('innerdia');
    expIfNotDef('leglength');
    expIfNotDef('wrapspace');
    var e = '';
    var x;
    b = defaultCoil();
    for (x in c) {
        if (c[x] !== '' && c[x] !== b[x]) {
            e = e + '&' + x + '=' + c[x]
        }
    }
    updateQs(e);
    return
}

function wraps() {
    $('.cd-calcicon').show();
    clearTimeout(aux.calculatingtimeout);
    aux.calculatingtimeout = setTimeout(function() {
                                        $('.cd-calcicon').fadeOut(250)
                                        }, 300);
    var f;
    var g;
    var h = coil.dia;
    if (coil.profile === 'ribbon') {
        f = coil.innerdia + (coil.thickness * 2);
        g = coil.innerdia + coil.thickness
    } else {
        if (coil.twistpitch !== 0) {
            if (coil.strands === 2) {
                h *= 2
            } else if (coil.strands === 3) {
                h /= (2 * Math.sqrt(3)) - 3
            } else if (coil.strands === 4) {
                h *= 1 + Math.sqrt(2)
            }
        }
        f = coil.innerdia + (h * 2);
        g = coil.innerdia + h
    }
    var i;
    if (coil.useNaDia) {
        i = (g * Math.PI)
    } else {
        i = (f * Math.PI)
    }
    var j;
    var k;
    if (coil.profile === 'ribbon') {
        k = coil.wrapspace + coil.width
    } else if (coil.twistpitch === 0) {
        k = (coil.wrapspace + coil.dia) * coil.strands
    } else {
        k = coil.wrapspace + h
    }
    j = Math.sqrt((i * i) + (k * k));
    var l = coil.targetr / getRfactor(coil);
    var m;
    if (coil.profile === 'ribbon') {
        m = Math.atan2(coil.wrapspace + coil.width, i) * 180 / Math.PI
    } else if (coil.twistpitch === 0) {
        m = Math.atan2((coil.wrapspace + coil.dia) * coil.strands, i) * 180 / Math.PI
    } else {
        m = Math.atan2(coil.wrapspace + h, i) * 180 / Math.PI
    }
    var n = coil.wirelength - coil.leglength;
    var o = n / j;
    var p = Math.round(o);
    var q;
    if (coil.profile === 'ribbon') {
        q = ((o - 1) * (coil.wrapspace + coil.width)) + coil.width
    } else if (coil.twistpitch === 0) {
        q = ((o - 1) * ((coil.wrapspace + coil.dia) * coil.strands)) + (coil.dia * coil.strands)
    } else {
        q = ((o - 1) * (coil.wrapspace + h)) + h
    }
    coil.legloss = coil.leglength / coil.wirelength;
    var r = coil.legloss * l;
    var s = (1 - coil.legloss) * l;
    var t;
    var u;
    if (coil.showSi) {
        t = 'mm';
        u = 1
    } else {
        t = 'in';
        u = 3
    }
    $('.summary-unmlen').html(t);
    $('#summary-wirelength .summary-res').html(ds(coil.wirelength, u, u));
    $('#summary-numberOfWraps .summary-res').html(round(o, 2));
    var v = ((p / o) * s) + r;
    var w = (((parseInt(o + 1, 10) - 0.5) / o) * s) + r;
    $('#summary-roundedNumberOfWraps .summary-res').html(p);
    $('#summary-roundedNumberOfWraps .summary-ohm').html(round(v));
    $('#summary-halfRoundedNumberOfWraps .summary-res').html(parseInt(o + 1, 10) + '/' + parseInt(o, 10));
    $('#summary-halfRoundedNumberOfWraps .summary-ohm').html(round(w));
    if (coil.setup !== 's') {
        $('#summary-coilresistance .summary-res').html(round(l, 2))
    }
    var x;
    x = $('#summary-legloss');
    x.attr('title', '').removeClass('warning danger');
    if (coil.legloss > 1) {
        x.attr('title', '線腳過長,請修改參數後重試');
        x.addClass('danger')
    } else if (coil.legloss > 0.3) {
        x.attr('title', '線腳損耗熱量過多,請更改參數後重試');
        x.addClass('warning')
    } else if (coil.legloss > 0.2) {
        x.attr('title', '線腳損耗熱量巨多,請更改參數後重試')
    }
    x.find('td.summary-res').html(round(coil.legloss * 100, 1));
    x = $('#summary-coillength .summary-res');
    if (n > 0) {
        x.html(ds(n, u, u))
    } else {
        x.html('<span class="label label-danger" title="線腳太長,無法繞線圈，請更改參數後重試">' + ds(n, u) + '</span>')
    }
    $('#summary-outerdia .summary-res').html(ds(f, u + 1, u + 1));
    $('#summary-neutralaxisdia .summary-res').html(ds(g, u + 1, u + 1));
    $('#summary-loopcircum .summary-res').html(ds(i, u + 1, u + 1));
    $('#summary-helixangle .summary-res').html(round(m, 1));
    $('#summary-looplength .summary-res').html(ds(j, u + 1, u + 1));
    $('#summary-coilwidth .summary-res').html(ds(q, u, u));
    var y = 1 - coil.legloss;
    var z;
    if (coil.profile === 'ribbon') {
        z = (2 * coil.width) + (2 * coil.thickness)
    } else {
        z = coil.dia * Math.PI * coil.strands
    }
    var A = coil.wirelength * z * coil.twistfactor;
    var B = A * y;
    var C = $('#hfnw').val();
    if (isNaN(C)) {
        C = 1;
        $('#hfnw').val(C)
    } else {
        C = readNumber(C)
    }
    var D = Math.sqrt(C * coil.targetr);
    var E = C / A;
    var F = D / A;
    var G = findXArea(coil);
    var H = G * coil.wirelength * coil.twistfactor;
    var I = findMat(coil.mat).dens;
    var J = H * I;
    var K = findMat(coil.mat).hc;
    var L = K * J;
    coil.tmpHeatCapacity = L;
    coil.tmpSurfaceArea = A;
    var M = 1.5;
    var N = (A * 0.085) + (L * 0.16);
    var O = Math.sqrt(N * coil.targetr);
    var P = Math.sqrt(N * M * coil.targetr);
    if (coil.setup === 'dp' || coil.setup === 'ds') {
        E /= 2;
        N *= 2
    } else if (coil.setup === 'tp' || coil.setup === 'ts') {
        E /= 3;
        N *= 3
    } else if (coil.setup === 'qp' || coil.setup === 'qs' || coil.setup === 'qsp') {
        E /= 4;
        N *= 4
    } else if (coil.setup === '5p') {
        E /= 5;
        N *= 5
    } else if (coil.setup === '6p') {
        E /= 6;
        N *= 6
    } else if (coil.setup === '7p') {
        E /= 7;
        N *= 7
    } else if (coil.setup === '8p') {
        E /= 8;
        N *= 8
    }
    if (coil.setup === 'ds' || coil.setup === 'qsp') {
        O *= 2;
        P *= 2
    } else if (coil.setup === 'ts') {
        O *= 3;
        P *= 3
    } else if (coil.setup === 'qs') {
        O *= 4;
        P *= 4
    }
    if (coil.showSi) {
        $('#summary-surfacearea .summary-res').html(round(A, 2));
        $('#summary-coilarea .summary-res').html(round(B, 2));
        $('#summary-wireheatfluxp .summary-res').html(round(E * 1000, 0));
        $('#summary-xarea .summary-res').html(round(G, 3));
        $('#summary-wirevolume .summary-res').html(round(H, 3));
        $('#summary-density .summary-res').html(round(I, 3));
        $('#summary-wiremass .summary-res').html(round(J, 3))
    } else {
        $('#summary-surfacearea .summary-res').html(round(A / in2, 5));
        $('#summary-coilarea .summary-res').html(round(B / in2, 5));
        $('#summary-wireheatfluxp .summary-res').html(round(E * 1000 * in2, 0));
        $('#summary-wireheatfluxu .summary-res').html(round(F * 1000 * in2, 0));
        $('#summary-xarea .summary-res').html(round(G / in2, 6));
        $('#summary-wirevolume .summary-res').html(round(H / in3, 6));
        $('#summary-density .summary-res').html(round(I * 0.578036672, 3));
        $('#summary-wiremass .summary-res').html(round(g2gr(J / 1000), 7))
    }
    $('#summary-heatcapacity .summary-res').html(round(L, 2));
    $('#summary-recommendp .summary-res').html(round(N, 0) + ' &ndash; ' + round(N * M, 0));
    $('#summary-recommendu .summary-res').html(round(O, 1) + ' &ndash; ' + round(P, 1));
    
    function fargHeatFlux(a) {
        var b = $('#heaticon');
        var d = ['0098E5', '00ACE4', '00C0E3', '00D4E2', '00E1D9', '00E0C3', '00DFAD', '00DE98', '00DC6E', '00DA44', '33D400', '59D200', '7FD000', 'A4CE00', 'B7CD00', 'C9CC00', 'CBBC00', 'CAA800', 'C99400', 'C88100', 'C76E00', 'C65B00', 'C54800', 'C43500', 'C32300', 'C21100', 'C10000', 'C00012', 'BF0024', 'BF0035'];
        var e = Math.round(a * 80) - 4;
        if (e < 1) {
            e = 1
        }
        if (e > d.length - 1) {
            e = d.length - 1
        }
        var c = d[e];
        b.css('background-color', '#' + c);
        return
    }
    fargHeatFlux(E);
    return
}

function setupFracInput(a) {
    var b = a.find('input').filter(':first');
    var c = a.attr('id');
    var d = [];
    d.push('<div class="fractionmenu" id="' + c + '-fractionmenu">');
    d.push('<span class="btn btn-default btn-xs pull-right" id="' + c + '-fractionbtn"><sup>x</sup>/<sub>y</sub>"</span>');
    d.push('<div class="fractionpresets" style="display:none;" id="' + c + 'scd-fractionpresets">');
    d.push('<div class="btn-group-justified">');
    d.push('<span class="btn btn-default btn-xs" data-cd-im="0.25">1/4"</span>');
    d.push('<span class="btn btn-default btn-xs" data-cd-im="0.125">1/8"</span>');
    d.push('<span class="btn btn-default btn-xs" data-cd-im="0.375">3/8"</span>');
    d.push('<span class="btn btn-default btn-xs" data-cd-im="0.0625">1/16"</span>');
    d.push('<span class="btn btn-default btn-xs" data-cd-im="0.1875">3/16"</span>');
    d.push('<span class="btn btn-default btn-xs" data-cd-im="0.3125">5/16"</span>');
    d.push('</div>');
    d.push('<div class="btn-group-justified">');
    d.push('<span class="btn btn-default btn-xs" data-cd-im="0.03125">1/32""</span>');
    d.push('<span class="btn btn-default btn-xs" data-cd-im="0.09375">3/32"</span>');
    d.push('<span class="btn btn-default btn-xs" data-cd-im="0.15625">5/32"</span>');
    d.push('<span class="btn btn-default btn-xs" data-cd-im="0.21875">7/32"</span>');
    d.push('<span class="btn btn-default btn-xs" data-cd-im="0.28125">9/32"</span>');
    d.push('<span class="btn btn-default btn-xs" data-cd-im="0.34375">11/32"</span>');
    d.push('</div>');
    d.push('<div class="btn-group-justified">');
    d.push('<span class="btn btn-default btn-xs" data-cd-im="0.015625"><sup>1</sup>/<sub>64</sub>"</span>');
    d.push('<span class="btn btn-default btn-xs" data-cd-im="0.046875"><sup>3</sup>/<sub>64</sub>"</span>');
    d.push('<span class="btn btn-default btn-xs" data-cd-im="0.078125"><sup>5</sup>/<sub>64</sub>"</span>');
    d.push('<span class="btn btn-default btn-xs" data-cd-im="0.109375"><sup>7</sup>/<sub>64</sub>"</span>');
    d.push('<span class="btn btn-default btn-xs" data-cd-im="0.140625"><sup>9</sup>/<sub>64</sub>"</span>');
    d.push('<span class="btn btn-default btn-xs" data-cd-im="0.171875"><sup>11</sup>/<sub>64</sub>"</span>');
    d.push('</div>');
    d.push('<div class="btn-group-justified">');
    d.push('<span class="btn btn-default btn-xs" data-cd-im="0.203125"><sup>13</sup>/<sub>64</sub>"</span>');
    d.push('<span class="btn btn-default btn-xs" data-cd-im="0.234375"><sup>15</sup>/<sub>64</sub>"</span>');
    d.push('<span class="btn btn-default btn-xs" data-cd-im="0.265625"><sup>17</sup>/<sub>64</sub>"</span>');
    d.push('<span class="btn btn-default btn-xs" data-cd-im="0.296875"><sup>19</sup>/<sub>64</sub>"</span>');
    d.push('<span class="btn btn-default btn-xs" data-cd-im="0.328125"><sup>21</sup>/<sub>64</sub>"</span>');
    d.push('<span class="btn btn-default btn-xs" data-cd-im="0.359375"><sup>23</sup>/<sub>64</sub>"</span>');
    d.push('</div>');
    d.push('</div>');
    d.push('</div>');
    var e = a.find('.help-block').filter(':first');
    if (e.length === 0) {
        a.append(d.join(''))
    } else {
        e.before(d.join(''))
    }
    var f = $('#' + c + 'scd-fractionpresets').find('.btn');
    f.click(function() {
            f.removeClass('active');
            $(this).addClass('active');
            var p = $(this).attr('data-cd-im');
            if (coil.showSi) {
            p = round(in2mm(p), 3)
            } else {
            p = $(this).attr('data-cd-im')
            }
            b.val(p).change();
            wraps();
            gad('coil', 'fracinputdiv-' + a.attr('id'), $(this).text())
            });
    $('#' + c + '-fractionbtn').click(function() {
                                      if (!$(this).hasClass('active')) {
                                      $('#' + c + 'scd-fractionpresets').slideDown()
                                      } else {
                                      $('#' + c + 'scd-fractionpresets').slideUp()
                                      }
                                      $(this).toggleClass('active')
                                      });
    $('#' + c).change(function() {
                      f.removeClass('active')
                      });
    return
}

function writeDia() {
    if (coil.dia === '') {
        $('#cd-dia').val('').parents('.form-group').addClass('has-warning')
    } else {
        $('#cd-dia').val(ds(coil.dia, 3, 5)).parents('.form-group').removeClass('has-warning')
    }
    return
}

function ccWrite() {
    $('#cd-material').val(coil.mat);
    $('#cd-profile').val(coil.profile);
    if (coil.profile === 'round') {
        $('.cd-wiredimensions').hide();
        $('.cd-wirediameter').show();
        $('.cd-twistpitch').hide();
        $('.cd-strands').hide()
    } else if (coil.profile === 'roundmulti') {
        $('.cd-wiredimensions').hide();
        $('.cd-wirediameter').show();
        $('#cd-twistpitch').val(ds(coil.twistpitch));
        $('.cd-twistpitch').show();
        $('.cd-twistpitch-output').html(coil.strands);
        $('.cd-strands').show()
    } else if (coil.profile === 'ribbon') {
        $('.cd-wirediameter').hide();
        $('.cd-wiredimensions').show();
        $('#cd-ribbonwidth').val(ds(coil.width, 4, 7));
        $('#cd-thickness').val(ds(coil.thickness, 4, 7));
        $('.cd-twistpitch').hide();
        $('.cd-strands').hide();
        $('cd-ribbonwidth').val(ds(coil.width, 5, 7));
        $('cd-thickness').val(ds(coil.thickness, 5, 7))
    } else {
        console.log('Unknown profile')
    }
    $('#cd-awg').val(coil.awg);
    if (coil.dia === '') {
        $('#cd-dia').val('').parents('.form-group').addClass('has-warning')
    } else {
        $('#cd-dia').val(ds(coil.dia)).parents('.form-group').removeClass('has-warning')
    }
    $('#cd-strands').val(coil.strands);
    $('#cd-setup').val(coil.setup);
    $('#cd-targetr').val(coil.targetr);
    $('#cd-innerdia').val(ds(coil.innerdia, 2, 3));
    $('#cd-leglength').val(ds(coil.leglength, 1, 3));
    $('#hfnw').val(coil.hfnw);
    $('#cd-wrapspace').val(ds(coil.wrapspace));
    $('#useneutralaxis').prop('checked', (coil.useNaDia === true));
    return
}

function updateUnits() {
    if (coil.showSi) {
        $('.cd-im').hide();
        $('.cd-si').show();
        $('#cd-units-si').prop('checked', true)
    } else {
        $('.cd-si').hide();
        $('.cd-im').show();
        $('#cd-units-im').prop('checked', true)
    }
    return
}

function toggleUi(a, b, c, d, e) {
    clearTimeout(e);
    if (c) {
        if (d) {
            a.show('blind')
        } else {
            a.show()
        } if (b) {
            b.addClass('active')
        }
    } else {
        if (d) {
            a.hide('blind')
        } else {
            a.hide()
        } if (b) {
            b.removeClass('active')
        }
    }
    return
}

function showMulticoilInfo() {
    $('.cd-setup-s').hide();
    $('.cd-setup-dp').hide();
    $('.cd-setup-ds').hide();
    $('.cd-setup-tp').hide();
    $('.cd-setup-ts').hide();
    $('.cd-setup-qp').hide();
    $('.cd-setup-qs').hide();
    $('.cd-setup-qsp').hide();
    if (coil.setup === 's') {
        $('.cd-setup-multi').hide();
        $('.cd-setup-single').show();
        $('#cd-len-number').html('').hide()
    } else {
        $('.cd-setup-single').hide();
        $('.cd-setup-multi').show();
        if (coil.setup === 'dp') {
            $('.cd-setup-dp').show()
        } else if (coil.setup === 'ds') {
            $('.cd-setup-ds').show()
        } else if (coil.setup === 'tp') {
            $('.cd-setup-tp').show()
        } else if (coil.setup === 'ts') {
            $('.cd-setup-ts').show()
        } else if (coil.setup === 'qp') {
            $('.cd-setup-qp').show()
        } else if (coil.setup === 'qs') {
            $('.cd-setup-qs').show()
        } else if (coil.setup === 'qsp') {
            $('.cd-setup-qsp').show()
        }
        $('#cd-len-number').html(' (&times; ' + $('#cd-setup').find('option:selected').attr('data-setup-wires') + ' wires)').show()
    }
    return
}

function toggleBasicAdvanced(a) {
    toggleUi($('.cd-advanced'), $('#cd-basicadvanced'), coil.advanced, a, coil.showbasicanimtimeout);
    toggleUi($('.cd-adv'), false, coil.advanced, false);
    return
}

function importCoil(a) {
    var x;
    for (x in a) {
        if (a.hasOwnProperty(x)) {
            if (x === 'll') {
                a.leglength = a.ll
            }
            if (x === 'id') {
                a.innerdia = a.id
            }
            if (x === 'r') {
                a.targetr = a.r
            }
            if (x === 's') {
                a.setup = a.s
            }
            if (x === 'a') {
                a.advanced = a.a
            }
            if (x === 'si') {
                a.showSi = a.si
            }
            if (x === 'p') {
                a.profile = a.p
            }
            if (x === 'tp') {
                a.twistpitch = a.tp
            }
            if (x === 'str') {
                a.strands = a.str
            }
            if (x === 'wl') {
                a.wirelength = a.wl
            }
            if (x === 'ws') {
                a.wrapspace = a.ws
            }
        }
    }
    for (x in a) {
        if (a.hasOwnProperty(x)) {
            if (a[x] === '') {} else if (a[x] === 'true') {
                coil[x] = true
            } else if (a[x] === 'false') {
                coil[x] = false
            } else if (!isNaN(a[x])) {
                coil[x] = readNumber(a[x])
            } else {
                coil[x] = a[x]
            }
        }
    }
    updateUnits();
    ccWrite();
    toggleBasicAdvanced(true);
    if (a.awg) {
        coil.dia = awg2mm(a.awg);
        writeDia()
    } else if (a.dia) {
        coil.awg = '';
        $('#cd-awg').val('');
        writeDia()
    }
    findResistivity(coil);
    findTwist(coil);
    if (a.rstv) {
        coil.mat = 'c';
        $('#cd-material').val(coil.mat);
        coil.rstv = a.rstv;
        writeRstv(coil)
    }
    if (a.wirelength) {
        coil.wirelength = a.wirelength;
        $('#cd-len').val(ds(coil.wirelength))
    } else {
        wireLength(coil)
    }
    wraps();
    return
}

function mm2awg(d) {
    function finn(a, b) {
        var c, awgmm, lt, hit, gt;
        for (c = a; c < b; c++) {
            awgmm = awg2mm(c);
            if (awgmm > d) {
                lt = c
            } else if (awgmm === d) {
                hit = c
            } else if (awgmm < d) {
                gt = c;
                return {
                lt: lt,
                hit: hit,
                gt: gt
                }
            }
        }
        return
    }
    var e = finn(-50, 500);
    if (e === undefined) {
        e = finn(-5000, 50000)
    }
    return e
}

function findClosestGauge(a, b, c, d) {
    var e, res = {};
    if (d === undefined) {
        d = 0
    }
    if (c === undefined) {
        c = 1
    }
    for (e = 0; e < a.length; e++) {
        if (a[e][c] > b) {
            res.lt = a[e][d]
        } else if (a[e][c] === b) {
            res = {};
            res.hit = a[e][d];
            return res
        }
        if (a[e][c] < b) {
            res.gt = a[e][d];
            return res
        }
    }
    return res
}

function findGauge(a, b, c, d) {
    var e;
    if (c === undefined) {
        c = 0
    }
    if (d === undefined) {
        d = 1
    }
    for (e = 0; e < a.length; e++) {
        if (String(a[e][c]) === String(b)) {
            return a[e][d]
        }
    }
    return
}

function writeGaugeResult(a, b, c) {
    if (c === undefined) {
        c = 'GA'
    }
    var d, tmp1, tmp2;
    d = findClosestGauge(b, mm2in(a.mm));
    var x = [];
    if (d.hit !== undefined) {
        x.push('<strong>' + d.hit + ' ' + c + '</strong>')
    } else if (d.lt !== undefined && d.gt !== undefined) {
        tmp1 = findGauge(b, d.gt);
        tmp2 = findGauge(b, d.lt);
        var e = (100 - ((a.mm / in2mm(tmp1)) * 100));
        var f = (100 - ((a.mm / in2mm(tmp2)) * 100));
        x.push('<i>between</i>');
        x.push('<br />');
        x.push('<strong>' + d.gt + ' ' + c + '</strong> (');
        if (a.useUnit === 'mm') {
            x.push(round(in2mm(tmp1), a.tod))
        } else {
            x.push(round(tmp1, a.tod))
        }
        x.push(' ' + a.useUnit + ')');
        x.push(' <span class="label label-');
        if (Math.abs(e) < Math.abs(f)) {
            x.push('success')
        } else {
            x.push('default')
        }
        x.push('">');
        if (e > 0) {
            x.push('+')
        }
        x.push(round(e));
        x.push('%</span>');
        x.push('<br /><i>and</i></br>');
        x.push('<strong>' + d.lt + ' ' + c + '</strong> (');
        if (a.useUnit === 'mm') {
            x.push(round(in2mm(tmp2), a.tod))
        } else {
            x.push(round(tmp2, a.tod))
        }
        x.push(' ' + a.useUnit + ')');
        x.push(' <span class="label label-');
        if (Math.abs(f) < Math.abs(e)) {
            x.push('success')
        } else {
            x.push('default')
        }
        x.push('">');
        if (f > 0) {
            x.push('+')
        }
        x.push(round(f));
        x.push('%</span>')
    } else {
        if (d.lt !== undefined) {
            tmp1 = b[b.length - 1][1]
        } else {
            tmp1 = b[0][1]
        } if (a.useUnit === 'mm') {
            tmp1 = in2mm(tmp1)
        }
        if (d.lt !== undefined) {
            x.push('輸入的值太小, 最小爲' + round(tmp1, a.tod) + ' ' + a.useUnit)
        } else {
            x.push('輸入的值太大, 最大爲' + round(tmp1, a.tod) + ' ' + a.useUnit)
        }
    }
    return x.join('')
}

function xor(a, b) {
    if (a === true && b !== true) {
        return true
    }
    if (a !== true && b === true) {
        return true
    }
    return false
}

function runConverter(a) {
    var b = '';
    var c = [];
    var d = '';
    if (a.fromu === 'mm' || a.fromu === 'in') {
        a.useUnit = a.fromu
    } else if (coil.showSi) {
        a.useUnit = 'mm'
    } else {
        a.useUnit = 'in'
    } if (a.fromu === 'mm') {
        a.mm = a.fromv
    } else if (a.fromu === 'in') {
        a.mm = in2mm(a.fromv)
    } else if (a.fromu === 'awg') {
        a.mm = awg2mm(a.fromv)
    } else if (a.fromu === 'bwg') {
        a.mm = in2mm(findGauge(a.bwgArr, a.fromv))
    } else if (a.fromu === 'swg') {
        a.mm = in2mm(findGauge(a.swgArr, a.fromv))
    } else if (a.fromu === 'fracin') {
        a.mm = in2mm(findGauge(a.fracInArr, a.fromv))
    }
    if (a.tou === 'mm') {
        a.res = a.mm
    } else if (a.tou === 'in') {
        a.res = mm2in(a.mm)
    } else if (a.tou === 'awg') {
        a.res = mm2awg(a.mm);
        if (a.res === undefined) {
            b = 'Cannot compute'
        } else if (a.res.hit !== undefined) {
            b = 'AWG <strong>' + a.res.hit + '<strong>';
            if (a.res.hit < 1) {
                d = 'AWG(美國線規)在標准格式下小于1將不會返回'
            }
        } else {
            b = (function() {
                 var x = [];
                 x.push('<strong>' + a.res.gt + '</strong> AWG');
                 if (a.useUnit === 'in') {
                 x.push(' (' + round(awg2in(a.res.gt), a.tod) + ' in)')
                 } else {
                 x.push(' (' + round(awg2mm(a.res.gt), a.tod) + ' mm)')
                 }
                 x.push(' &mdash; ');
                 x.push('<strong>' + a.res.lt + '</strong> AWG');
                 if (a.useUnit === 'in') {
                 x.push(' (' + round(awg2in(a.res.lt), a.tod) + ' in)')
                 } else {
                 x.push(' (' + round(awg2mm(a.res.lt), a.tod) + ' mm)')
                 } if (a.res.gt < 1) {
                 d = '線規大于AWG1時返回負數,而非標准格式下的多個0'
                 }
                 return x.join('')
                 }())
        }
    } else if (a.tou === 'bwg') {
        b = writeGaugeResult(a, a.bwgArr, 'BWG')
    } else if (a.tou === 'swg') {
        b = writeGaugeResult(a, a.swgArr, 'SWG')
    } else if (a.tou === 'fracin') {
        b = writeGaugeResult(a, a.fracInArr, 'in')
    }
    if (b === '') {
        b = '<strong>' + round(a.res, a.tod) + ' ' + a.tou + '</strong>'
    }
    c.push(a.fromv + ' ');
    if (a.fromu === 'mm' || a.fromu === 'in') {
        c.push(a.fromu)
    } else if (a.fromu === 'fracin') {
        c.push(' in')
    } else {
        c.push(a.fromu.toUpperCase())
    } if (coil.showSi && a.tou === 'mm') {} else if (!coil.showSi && a.tou === 'in') {} else if (coil.showSi && a.fromu === 'in') {
        c.push(' (' + round(in2mm(a.fromv), a.tod) + ' mm)')
    } else if (!coil.showSi && a.fromu === 'mm') {
        c.push(' (' + round(mm2in(a.fromv), a.tod) + ' in)')
    } else if (a.fromu === 'awg') {
        if (a.useUnit === 'in') {
            c.push(' (' + round(awg2in(a.fromv), a.tod) + ' in)')
        } else {
            c.push(' (' + round(awg2mm(a.fromv), a.tod) + ' mm)')
        }
    } else if (a.fromu === 'bwg') {
        if (coil.showSi && a.tou !== 'mm') {
            c.push(' (' + round(in2mm(parseFloat(findGauge(a.bwgArr, a.fromv))), a.tod) + ' mm)')
        } else if (!coil.showSi && a.tou !== 'in') {
            c.push(' (' + round(findGauge(a.bwgArr, a.fromv), a.tod) + ' in)')
        }
    } else if (a.fromu === 'swg') {
        if (coil.showSi && a.tou !== 'mm') {
            c.push(' (' + round(in2mm(parseFloat(findGauge(a.swgArr, a.fromv))), a.tod) + ' mm)')
        } else if (!coil.showSi && a.tou !== 'in') {
            c.push(' (' + round(findGauge(a.swgArr, a.fromv), a.tod) + ' in)')
        }
    } else if (a.fromu === 'fracin') {
        if (coil.showSi && a.tou !== 'mm') {
            c.push(' (' + round(in2mm(parseFloat(findGauge(a.fracInArr, a.fromv))), a.tod) + ' mm)')
        } else if (!coil.showSi && a.tou !== 'in') {
            c.push(' (' + round(findGauge(a.fracInArr, a.fromv), a.tod) + ' in)')
        }
    }
    $('#conres').html('<small class="text-muted">' + c.join('') + ' =</small><br />' + b);
    if (d === '') {
        $('#connote').hide()
    } else {
        $('#connote').html('Note: ' + d).show()
    }
    return
}

function conFillSel(a) {
    var b, xx = [];
    for (b = 0; b < a.length; b++) {
        xx.push('<option value="' + a[b][0] + '">' + a[b][0] + '</option>')
    }
    return xx.join('')
}

function initConverter() {
    var b = coil.converter;
    b.bwgArr = [
                ['00000 (5/0)', 0.500],
                ['0000 (4/0)', 0.454],
                ['000 (3/0)', 0.425],
                ['00 (2/0)', 0.380],
                ['0', 0.340],
                ['1', 0.300],
                ['2', 0.284],
                ['3', 0.259],
                ['4', 0.238],
                ['5', 0.220],
                ['6', 0.203],
                ['6.5', 0.188],
                ['7', 0.180],
                ['7.5', 0.172],
                ['8', 0.165],
                ['8.5', 0.156],
                ['9', 0.148],
                ['9.5', 0.141],
                ['10', 0.134],
                ['10.5', 0.126],
                ['11', 0.120],
                ['11.5', 0.115],
                ['12', 0.109],
                ['12.5', 0.1],
                ['13', 0.095],
                ['13.5', 0.089],
                ['14', 0.083],
                ['14.5', 0.078],
                ['15', 0.072],
                ['15.5', 0.068],
                ['16', 0.065],
                ['16.5', 0.062],
                ['17', 0.058],
                ['17.5', 0.056],
                ['18', 0.049],
                ['18.5', 0.046],
                ['19', 0.042],
                ['19.5', 0.039],
                ['20', 0.03575],
                ['20.5', 0.343],
                ['21', 0.03225],
                ['21.5', 0.0303],
                ['22', 0.02825],
                ['22.5', 0.0263],
                ['23', 0.02525],
                ['23.5', 0.0233],
                ['24', 0.02225],
                ['24.5', 0.0213],
                ['25', 0.02025],
                ['25.5', 0.0193],
                ['26', 0.01825],
                ['27', 0.01625],
                ['28', 0.01425],
                ['29', 0.01325],
                ['30', 0.01225],
                ['31', 0.01025],
                ['32', 0.00925],
                ['33', 0.00825],
                ['34', 0.00725],
                ['35', 0.005],
                ['36', 0.004]
                ];
    b.fracArr = ['1/64', '1/32', '3/64', '1/16', '5/64', '3/32', '7/64', '1/8', '9/64', '5/32', '11/64', '3/16', '13/64', '7/32', '15/64', '1/4', '17/64', '9/32', '19/64', '5/16', '21/64', '11/32', '23/64', '3/8', '25/64', '13/32', '27/64', '7/16', '29/64', '15/32', '31/64', '1/2', '33/64', '17/32', '35/64', '9/16', '37/64', '19/32', '39/64', '5/8', '41/64', '21/32', '43/64', '11/16', '45/64', '23/32', '47/64', '3/4', '49/64', '25/32', '51/64', '13/16', '53/64', '27/32', '55/64', '7/8', '57/64', '29/32', '59/64', '15/16', '61/64', '31/32', '63/64'];
    b.fracInArr = [];
    (function() {
     var a, tmp;
     for (a = 0; a < b.fracArr.length; a++) {
     tmp = b.fracArr[a].split('/');
     b.fracInArr.push(['<sup>' + tmp[0] + '</sup>/<sub>' + tmp[1] + '</sub>', parseInt(tmp[0], 10) / parseInt(tmp[1], 10)])
     }
     }());
    b.fracInArr.reverse();
    b.swgArr = [
                ['7/0', 0.500],
                ['6/0', 0.464],
                ['5/0', 0.432],
                ['4/0', 0.400],
                ['3/0', 0.372],
                ['2/0', 0.348],
                ['0', 0.324],
                ['1', 0.300],
                ['2', 0.276],
                ['3', 0.252],
                ['4', 0.232],
                ['5', 0.212],
                ['6', 0.192],
                ['7', 0.176],
                ['8', 0.160],
                ['9', 0.144],
                ['10', 0.128],
                ['11', 0.116],
                ['12', 0.104],
                ['13', 0.092],
                ['14', 0.080],
                ['15', 0.072],
                ['16', 0.064],
                ['17', 0.056],
                ['18', 0.048],
                ['19', 0.040],
                ['20', 0.0369],
                ['21', 0.032],
                ['22', 0.028],
                ['23', 0.024],
                ['24', 0.022],
                ['25', 0.020],
                ['26', 0.018],
                ['27', 0.0164],
                ['28', 0.0148],
                ['29', 0.0136],
                ['30', 0.0124],
                ['31', 0.0116],
                ['32', 0.0108],
                ['33', 0.0100],
                ['34', 0.0092],
                ['35', 0.0084],
                ['36', 0.0076],
                ['37', 0.0068],
                ['38', 0.0060],
                ['39', 0.0052],
                ['40', 0.0048],
                ['41', 0.0044],
                ['42', 0.004],
                ['43', 0.0036],
                ['44', 0.0032],
                ['45', 0.0028],
                ['46', 0.0024],
                ['47', 0.0020],
                ['48', 0.0016],
                ['49', 0.0012],
                ['50', 0.0010]
                ];
    b.fromv = 1;
    b.fromu = 'in';
    b.fromvStep = 0.001;
    b.tou = 'mm';
    b.tod = 3;
    $('#confromu').val(b.fromu).change(function() {
                                       b.fromu = $(this).val();
                                       $('#confromvsel').hide();
                                       $('#confromv').show();
                                       b.fromv = $('#confromv').val();
                                       if (b.fromu === 'awg') {
                                       b.fromvStep = 1
                                       } else if (b.fromu === 'in') {
                                       b.fromvStep = 0.001
                                       } else if (b.fromu === 'bwg') {
                                       $('#confromv').hide();
                                       $('#confromvsel').show().html(conFillSel(b.bwgArr));
                                       b.fromv = $('#confromvsel').val()
                                       } else if (b.fromu === 'swg') {
                                       $('#confromv').hide();
                                       $('#confromvsel').show().html(conFillSel(b.swgArr));
                                       b.fromv = $('#confromvsel').val()
                                       } else if (b.fromu === 'fracin') {
                                       $('#confromv').hide();
                                       $('#confromvsel').show().html(conFillSel(b.fracInArr));
                                       b.fromv = $('#confromvsel').val()
                                       } else {
                                       b.fromvStep = 0.1
                                       }
                                       $('#confromv').attr('step', b.fromvStep);
                                       runConverter(b);
                                       gad('coil-con', 'fromu', b.fromu)
                                       });
    $('#confromv').val(b.fromv).attr('step', b.fromvStep).change(function() {
                                                                 b.fromv = readNumber($(this).val());
                                                                 if (isNaN(b.fromv)) {
                                                                 b.fromv = 1;
                                                                 $(this).val(b.fromv)
                                                                 }
                                                                 runConverter(b);
                                                                 gad('coil-con', 'fromv', b.fromv)
                                                                 });
    $('#confromvsel').change(function() {
                             b.fromv = $(this).val();
                             runConverter(b);
                             gad('coil-con', 'fromvsel', b.fromv)
                             });
    $('#contou').val(b.tou).change(function() {
                                   b.tou = $(this).val();
                                   runConverter(b);
                                   gad('coil-con', 'tou', b.tou)
                                   });
    $('#contod').val(b.tod).change(function() {
                                   b.tod = parseInt($(this).val(), 10);
                                   if (isNaN(b.tod)) {
                                   b.tod = 3;
                                   $(this).val(b.tod)
                                   }
                                   runConverter(b);
                                   gad('coil-con', 'tod', b.tod)
                                   });
    $('.cd-units').change(function() {
                          runConverter(b)
                          });
    runConverter(b);
    gad('coil-con', 'init');
    return
}

function validerTwistPitch(a) {
    var b = $('#cd-twistpitch').parents('.form-group');
    var c = b.find('.tpwarn');
    if (a.twistpitch <= a.dia && a.twistpitch !== 0) {
        b.addClass('has-error');
        if (c.length === 0) {
            b.append('<span class="label label-danger tpwarn">扭矩應大于0，且扭矩大于線徑</span>')
        }
    } else {
        b.removeClass('has-error').find('.tpwarn').remove()
    }
    return
}

function populateMatSelect(a) {
    var b, xx = [];
    for (b = 0; b < a.length; b++) {
        xx.push('<option value="' + a[b].id + '">' + a[b].nm + '</option>')
    }
    xx.push('<option value="c">自定義電阻</option>');
    return xx.join('')
}
$(document).ready(function() {
                  materials = $.parseJSON(stripComments($('#materials').text()));
                  $('#cd-material').html(populateMatSelect(materials));
                  $('#cd-basicadvanced').click(function() {
                                               coil.advanced = !coil.advanced;
                                               toggleBasicAdvanced(true);
                                               if ($('#cd-int-export').is(':visible')) {
                                               exportSettings()
                                               }
                                               gad('coil', 'advanced', coil.advanced)
                                               }).mouseenter(function() {
                                                             $('.cd-advanced').addClass('bg-warning')
                                                             }).mouseleave(function() {
                                                                           $('.cd-advanced').removeClass('bg-warning')
                                                                           });
                  $('.cd-units').change(function() {
                                        coil.showSi = (!coil.showSi);
                                        updateUnits();
                                        ccWrite();
                                        writeDia();
                                        writeRstv(coil);
                                        findResistivity(coil);
                                        findTwist(coil);
                                        wireLength(coil);
                                        wraps();
                                        gad('coil', 'showsi', coil.showSi)
                                        });
                  $('#cd-material').change(function() {
                                           coil.mat = $(this).val();
                                           if (coil.mat === 'c') {
                                           coil.advanced = true;
                                           toggleBasicAdvanced(true)
                                           }
                                           writeDia();
                                           findResistivity(coil);
                                           findTwist(coil);
                                           wireLength(coil);
                                           wraps();
                                           gad('coil', 'mat', coil.mat)
                                           });
                  $('#cd-profile').change(function() {
                                          coil.profile = $(this).val();
                                          if (coil.profile === 'round') {
                                          coil.strands = 1
                                          } else if (coil.profile === 'roundmulti') {
                                          if (coil.strands < 2) {
                                          coil.strands = 2
                                          }
                                          } else if (coil.profile === 'ribbon') {
                                          coil.strands = 1
                                          } else {
                                          console.log('Unknown profile')
                                          }
                                          ccWrite();
                                          writeDia();
                                          findResistivity(coil);
                                          findTwist(coil);
                                          wireLength(coil);
                                          wraps();
                                          gad('coil', 'profile', $(this).val())
                                          });
                  $('#cd-awg').change(function() {
                                      var a = parseInt($(this).val(), 10);
                                      if (a < -10) {
                                      a = -10;
                                      $(this).val(a)
                                      } else if (a > 100) {
                                      a = 100;
                                      $(this).val(a)
                                      }
                                      $(this).parents('.form-group').removeClass('has-warning');
                                      coil.awg = a;
                                      coil.dia = awg2mm(coil.awg);
                                      writeDia();
                                      findResistivity(coil);
                                      findTwist(coil);
                                      wireLength(coil);
                                      wraps();
                                      validerTwistPitch(coil);
                                      gad('coil', 'awg', coil.awg)
                                      });
                  $('#cd-dia').change(function() {
                                      clearTimeout(coil.keyuptimeout);
                                      coil.dia = readNumber($(this).val());
                                      if (!coil.showSi) {
                                      coil.dia = in2mm(coil.dia)
                                      }
                                      $(this).parents('.form-group').removeClass('has-warning');
                                      coil.awg = '';
                                      $('#cd-awg').val('').parents('.form-group').addClass('has-warning');
                                      findResistivity(coil);
                                      findTwist(coil);
                                      wireLength(coil);
                                      wraps();
                                      validerTwistPitch(coil);
                                      gad('coil', 'dia', coil.dia)
                                      });
                  $('#cd-strands').change(function() {
                                          coil.strands = parseInt($(this).val(), 10);
                                          if (coil.twistpitch !== 0 && coil.strands > 4) {
                                          coil.strands = 4;
                                          $(this).val(4)
                                          }
                                          findResistivity(coil);
                                          findTwist(coil);
                                          wireLength(coil);
                                          wraps();
                                          gad('coil', 'pitch', coil.twistpitch);
                                          $('.cd-twistpitch-output').html(coil.strands)
                                          });
                  $('#cd-twistpitch').change(function() {
                                             coil.twistpitch = readNumber($(this).val());
                                             if (!coil.showSi) {
                                             coil.twistpitch = in2mm(coil.twistpitch)
                                             }
                                             if (coil.twistpitch !== 0 && coil.strands > 4) {
                                             coil.strands = 4;
                                             $('#cd-strands').val(4)
                                             }
                                             findResistivity(coil);
                                             findTwist(coil);
                                             wireLength(coil);
                                             wraps();
                                             validerTwistPitch(coil);
                                             gad('coil', 'pitch', coil.twistpitch)
                                             });
                  
                  function ribbonCheck() {
                  var a = $('.cd-wiredimensions');
                  if (coil.width < coil.thickness) {
                  a.addClass('has-warning');
                  $('#thicknesswarning').show()
                  } else {
                  a.removeClass('has-warning');
                  $('#thicknesswarning').hide()
                  }
                  return
                  }
                  $('#cd-ribbonwidth').change(function() {
                                              coil.width = readNumber($(this).val());
                                              if (!coil.showSi) {
                                              coil.width = in2mm(coil.width)
                                              }
                                              ribbonCheck();
                                              findResistivity(coil);
                                              wireLength(coil);
                                              wraps();
                                              gad('coil', 'ribbonwidth', coil.width)
                                              });
                  $('#cd-thickness').change(function() {
                                            coil.thickness = readNumber($(this).val());
                                            if (!coil.showSi) {
                                            coil.thickness = in2mm(coil.thickness)
                                            }
                                            ribbonCheck();
                                            findResistivity(coil);
                                            wireLength(coil);
                                            wraps();
                                            gad('coil', 'thickness', coil.thickness)
                                            });
                  $('#thicknesswarning .btn').click(function() {
                                                    var a = coil.thickness;
                                                    coil.thickness = coil.width;
                                                    coil.width = a;
                                                    ccWrite();
                                                    ribbonCheck();
                                                    findResistivity(coil);
                                                    wireLength(coil);
                                                    wraps()
                                                    });
                  $('#cd-setup').change(function() {
                                        coil.setup = $(this).val();
                                        showMulticoilInfo();
                                        findResistivity(coil);
                                        findTwist(coil);
                                        wireLength(coil);
                                        wraps();
                                        gad('coil', 'setup', coil.setup)
                                        });
                  $('#cd-targetr').change(function() {
                                          clearTimeout(coil.keyuptimeout);
                                          coil.targetr = readNumber($(this).val());
                                          findResistivity(coil);
                                          findTwist(coil);
                                          wireLength(coil);
                                          wraps();
                                          gad('coil', 'targetr', coil.targetr)
                                          });
                  $('#cd-rstv').change(function() {
                                       clearTimeout(coil.keyuptimeout);
                                       coil.rstv = readNumber($(this).val());
                                       coil.mat = 'c';
                                       $('#cd-material').val(coil.mat);
                                       if (!coil.showSi) {
                                       coil.rstv = mm2in(coil.rstv)
                                       }
                                       wireLength(coil);
                                       wraps();
                                       gad('coil', 'rstv', coil.rstv)
                                       });
                  $('#cd-len').change(function() {
                                      clearTimeout(coil.keyuptimeout);
                                      coil.wirelength = readNumber($(this).val());
                                      if (!coil.showSi) {
                                      coil.wirelength = in2mm(coil.wirelength)
                                      }
                                      wraps();
                                      gad('coil', 'wirelength', coil.wirelength)
                                      });
                  $('#cd-leglength').change(function() {
                                            clearTimeout(coil.keyuptimeout);
                                            coil.leglength = readNumber($(this).val());
                                            if (!coil.showSi) {
                                            coil.leglength = in2mm(coil.leglength)
                                            }
                                            wraps();
                                            gad('coil', 'leglength', coil.leglength)
                                            });
                  $('#cd-wrapspace').change(function() {
                                            clearTimeout(coil.keyuptimeout);
                                            coil.wrapspace = readNumber($(this).val());
                                            if (!coil.showSi) {
                                            coil.wrapspace = in2mm(coil.wrapspace)
                                            }
                                            wraps();
                                            gad('coil', 'wrapspace', coil.wrapspace)
                                            });
                  $('#hfnw').change(function() {
                                    coil.hfnw = $(this).val();
                                    if (isNaN(coil.hfnw)) {
                                    coil.hfnw = 10;
                                    $(this).val(coil.hfnw)
                                    } else {
                                    coil.hfnw = readNumber(coil.hfnw)
                                    }
                                    wraps();
                                    gad('coil', 'hfnw', round(coil.tmpSurfaceArea, 0) + ',' + round(coil.tmpHeatCapacity, 0) + ',' + coil.hfnw)
                                    });
                  $('#cd-innerdia').change(function() {
                                           clearTimeout(coil.keyuptimeout);
                                           coil.innerdia = readNumber($(this).val());
                                           if (!coil.showSi) {
                                           coil.innerdia = in2mm(coil.innerdia)
                                           }
                                           $('#cd-innerdia-presets .btn').removeClass('active');
                                           wraps();
                                           gad('coil', 'innerdia', coil.innerdia)
                                           });
                  $('#cc-resetdefaults').click(function() {
                                               coil = defaultCoil();
                                               updateUnits();
                                               ccWrite();
                                               writeDia();
                                               findResistivity(coil);
                                               findTwist(coil);
                                               wireLength(coil);
                                               wraps();
                                               toggleBasicAdvanced(true);
                                               ccMenuMsg('已加載默認配置', 'success');
                                               gad('coil', 'reset')
                                               });
                  $('.fractioninputdiv').each(function() {
                                              setupFracInput($(this))
                                              });
                  $('#useneutralaxis').click(function() {
                                             coil.useNaDia = ($(this).is(':checked'));
                                             wraps();
                                             exportSettings()
                                             });
                  $('input, select').change(function() {
                                            exportSettings()
                                            });
                  $('.btn').click(function() {
                                  exportSettings()
                                  });
                  toggleBasicAdvanced(false);
                  var b = getQueryStrings();
                  
                  function endOfInit() {
                  updateUnits();
                  toggleBasicAdvanced(true);
                  ccWrite();
                  writeDia();
                  findResistivity(coil);
                  findTwist(coil);
                  wireLength(coil);
                  wraps();
                  return
                  }
                  choo.showSi = false;
                  if (Object.keys(b).length > 0) {
                  importCoil(b);
                  ccMenuMsg('已保存配置導入成功', 'success');
                  gad('coil', 'importqs', JSON.stringify(b))
                  } else {
                  endOfInit()
                  }
                  showMulticoilInfo();
                  $('#rescontab a').click(function(e) {
                                          e.preventDefault();
                                          $(this).tab('show')
                                          });
                  $('#rescontab ul.nav li a[href="#converter"]').click(function() {
                                                                       if (coil.conInited !== true) {
                                                                       initConverter();
                                                                       coil.conInited = true
                                                                       }
                                                                       })
                  });